<?php
namespace Teknisa\CUS\Modules\Relatorio\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Cus00600 extends \Zeedhi\Framework\Controller\Report {

    protected $novoCodigo;
    protected $procedureConsumoPlanejado;
    protected $consplan;
    protected $nrorg;
    protected $cdfilial;
    protected $qr2Strategy;
    protected $birtStrategy;
    protected $nrConsPlan;
    protected $environment;

    public function __construct(ReportService $reportService, $novoCodigo, $consplan, $qr2Strategy, $birtStrategy, $servicoPorGrupo, StoredProcedure $procedureConsumoPlanejado, $environment) {
        $this->novoCodigo                = $novoCodigo;
        $this->consplan                  = $consplan;        
        $this->servicoPorGrupo           = $servicoPorGrupo;
        $this->procedureConsumoPlanejado = $procedureConsumoPlanejado;
        $this->qr2Strategy               = $qr2Strategy;
        $this->birtStrategy              = $birtStrategy;
        $this->reportService             = $reportService;
        $this->environment               = $environment;
        parent::__construct($reportService);
    }
    
    protected $reportMapping = array (
        'CUS00600' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping' => 
            array (
                'CDTIPOCUSTO' => 'P_NRTIPOCUSTO',   
                'NMTIPOCUSTO1' => 'P_NMCUSTO',
                'DTPERIINIC'   => 'P_DTPERIINIC',
                'DTPERIFINA'   => 'P_DTPERIFINA',
                'IDMATEPRIM'   => 'P_IDMATEPRIM',
                'IDNAOESTOC'   => 'P_IDNAOESTOC',
                'IDCONSPERC'   => 'P_IDCONSPERC',
                'IDCONSFIXO'   => 'P_IDCONSFIXO',
                'NRTAMACNIVEL' => 'P_NRTAMACNIVEL',
                'NRTAMCODIGO'  => 'P_NRTAMCODIGO',
                'CDFILIAL1'    => 'P_CDFILIAL',
                'LISTASERV'    => 'P_CDSERVFILI',
                'LISTASERV1'   => 'L_CDSERVFILI',
                'DTINICIAL'    => 'P_DATAINIC',
                'DTFINAL'      => 'P_DATAFINA',
                'NRCONSPLAN'   => 'P_NRCONSPLAN',
                'DTMESINIC'    => 'P_DTMESINIC',
                'DTMESFINA'    => 'P_DTMESFINA',
                'NRORG'        => 'P_NRORG'
            ),
        ),
        'CUS00601' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping'  => 
            array (
                'CDTIPOCUSTO'  => 'P_NRTIPOCUSTO',
                'NMTIPOCUSTO1'  => 'P_NMCUSTO',
                'DTPERIINIC'    => 'P_DTPERIINIC',
                'DTPERIFINA'    => 'P_DTPERIFINA',
                'IDMATEPRIM'    => 'P_IDMATEPRIM',
                'IDNAOESTOC'    => 'P_IDNAOESTOC',
                'IDCONSPERC'    => 'P_IDCONSPERC',
                'IDCONSFIXO'    => 'P_IDCONSFIXO',
                'NRTAMACNIVEL'  => 'P_NRTAMACNIVEL',
                'NRTAMCODIGO'   => 'P_NRTAMCODIGO',
                'CDFILIAL1'     => 'P_CDFILIAL',
                'LISTASERV'     => 'P_CDSERVFILI',
                'LISTASERV1'    => 'L_CDSERVFILI',
                'DTINICIAL'     => 'P_DATAINIC',
                'DTFINAL'       => 'P_DATAFINA',
                'NRCONSPLAN'    => 'P_NRCONSPLAN',
                'DTMESINIC'     => 'P_DTMESINIC',
                'DTMESFINA'     => 'P_DTMESFINA',
                'NRORG'         => 'P_NRORG'
            ),
        ),
        'CUS00602' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping'  => 
            array (
                'CDFILIAL1'     => 'P_CDFILIAL',
                'NRCONSPLAN'    => 'P_NRCONSPLAN',
                'NMTIPOCUSTO1'  => 'P_NMCUSTO',
                'CDTIPOCUSTO'  => 'P_NRTIPOCUSTO',
                'DTINICIAL'     => 'P_DATAINIC',
                'DTFINAL'       => 'P_DATAFINA',
                'DTPERIINIC'    => 'P_DTPERIINIC',
                'DTPERIFINA'    => 'P_DTPERIFINA',
                'IDMATEPRIM'    => 'P_IDMATEPRIM',
                'IDNAOESTOC'    => 'P_IDNAOESTOC',
                'IDCONSPERC'    => 'P_IDCONSPERC',
                'IDCONSFIXO'    => 'P_IDCONSFIXO',
                'DTMESINIC'     => 'P_DTMESINIC',
                'DTMESFINA'     => 'P_DTMESFINA',
                'LISTASERV'     => 'P_CDSERVFILI',
                'LISTASERV1'    => 'L_CDSERVFILI',
                'NRTAMACNIVEL'  => 'P_NRTAMACNIVEL',
                'NRTAMCODIGO'   => 'P_NRTAMCODIGO',
                'NRORG'         => 'P_NRORG'
            ),
        ),
        'CUS00603' => 
        array (
            'strategy' => 'BIRT',
            'parameterMapping'  => 
            array (
                'CDFILIAL1'     => 'P_CDFILIAL',
                'NRCONSPLAN'    => 'P_NRCONSPLAN',
                'NMTIPOCUSTO1'  => 'P_NMCUSTO',
                'CDTIPOCUSTO'  => 'P_NRTIPOCUSTO',
                'DTINICIAL'     => 'P_DATAINIC',
                'DTFINAL'       => 'P_DATAFINA',
                'DTPERIINIC'    => 'P_DTPERIINIC',
                'DTPERIFINA'    => 'P_DTPERIFINA',
                'IDMATEPRIM'    => 'P_IDMATEPRIM',
                'IDNAOESTOC'    => 'P_IDNAOESTOC',
                'IDCONSPERC'    => 'P_IDCONSPERC',
                'IDCONSFIXO'    => 'P_IDCONSFIXO',
                'DTMESINIC'     => 'P_DTMESINIC',
                'DTMESFINA'     => 'P_DTMESFINA',
                'LISTASERV'     => 'P_CDSERVFILI',
                'LISTASERV1'    => 'L_CDSERVFILI',
                'NRTAMACNIVEL'  => 'P_NRTAMACNIVEL',
                'NRTAMCODIGO'   => 'P_NRTAMCODIGO',
                'NRORG'         => 'P_NRORG'
            ),
        ),
        'cus00604' => 
        array (
            'strategy' => 'QR2',
            'parameterMapping'  =>
            array (
                'CDFILIAL1'     => 'P_CDFILIAL',
                'CDFILIAL2'     => 'P_CDFILICPLA2',
                'NMFILIAL2'     => 'P_NMFILIAL2',
                'NRCONSPLAN'    => 'P_NRCONSPLAN',
                'NMTIPOCUSTO1'  => 'P_NMCUSTO',
                'NMTIPOCUSTO2'  => 'P_NMCUSTO2',
                'CDTIPOCUSTO1'  => 'P_NRTIPOCUSTO',
                'CDTIPOCUSTO2'  => 'P_NRTIPOCUSTO2',
                'DTINICIAL'     => 'P_DATAINIC',
                'DTFINAL'       => 'P_DATAFINA',
                'DTPERIINIC'    => 'P_DTPERIINIC',
                'DTPERIFINA'    => 'P_DTPERIFINA',
                'IDMATEPRIM'    => 'P_IDMATEPRIM',
                'IDNAOESTOC'    => 'P_IDNAOESTOC',
                'IDCONSPERC'    => 'P_IDCONSPERC',
                'IDCONSFIXO'    => 'P_IDCONSFIXO',
                'DTMESINIC'     => 'P_DTMESINIC',
                'DTMESFINA'     => 'P_DTMESFINA',
                'LISTASERV'     => 'P_CDSERVFILI',
                'LISTASERV1'    => 'L_CDSERVFILI',
                'NRTAMACNIVEL'  => 'P_NRTAMACNIVEL',
                'NRTAMCODIGO'   => 'P_NRTAMCODIGO',
            ),
        ),
    );
    
    
    //Função para verificar se existe algum grupo sem seviços cadastrados.
    public function validaGrupoServico($nrorg, $cdfilial, $cdoperador, $cdgrupserv, $cdgrupfili = null) {

        $cdgrupserv = explode('_', $cdgrupserv);

        $gruposServico = array();
        $grupoVazio = array();
        $grupoValido = array();
        
            foreach ($cdgrupserv as $cdgrupservs) {
    
                $gruposServico = $this->servicoPorGrupo->retornaServicosPorGrupo($nrorg, $cdfilial, $cdoperador, $cdgrupservs, $cdgrupfili);
                
                if(!$gruposServico){
                    $grupoVazio[] =  $cdgrupservs;
                }else{
                    $grupoValido[] = $cdgrupservs;
                }
            }
        if($grupoValido) {
            return implode("_", $grupoValido);
        }else{
            throw new \Exception('Não existe Serviços cadastrado nesse grupo!');
            
        }   
       
    }
    

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $row['NMFILIAL2'] = $row['NMFILIAL_BKP'];
        
        if($row['REPORT_NAME'] !== 'cus00604') {
            $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
        }else {
            $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
        }
        
        $row['NMTIPOCUSTO1'] = substr($row['NMTIPOCUSTO1'], strpos($row['NMTIPOCUSTO1'], '|') + 2);
        $dateInicialFormatted = "";//\DateTime::createFromFormat('d/m/Y', $row['DTINICIAL']);
        $dateInicial          = $row['DTINICIAL'];
        $dateFinalFormatted   = "";//\DateTime::createFromFormat('d/m/Y', $row['DTFINAL']);
        $dateFinal            = $row['DTFINAL'];
        $cdfilial             = $row['CDFILIAL1'];
        $nrorg                = $this->environment->getNrOrg();
        $cdoperador           = $this->environment->getCdOperador();
        $idArquivo            = "T";
        $idProdutos           = $row['IDPRODUTOS'];
        $idSepPrato           = "N";
        $peAcrescimo          = 0;
        $idarredond           = "N";
        $cdProdutoIni         = " ";
        $cdgrupserv           = $row ['LISTAGRUPSERV'];
        
        $validaGrupoComServico = $this->validaGrupoServico($nrorg, $cdfilial, $cdoperador, $cdgrupserv);
        //Atribui somente grupos que tem serviços cadastrados a variavel $cdgrupserv.
        $cdgrupserv = $validaGrupoComServico;
        
        if ($row['LISTASERV'] == 'REQUEST') {
            $row['LISTASERV'] = $this->servicoPorGrupo->retornaServicosPorGrupo($nrorg, $cdfilial, $cdoperador, $cdgrupserv);
        }

        $row['LISTASERV1'] = $row['LISTASERV'];

        $nrConsPlan = $this->novoCodigo->getCodeByTableName('CONSPLAN'.$cdfilial, $nrorg);
       
        $this->procedureConsumoPlanejado->addParam(new Param('P_FILIAL',        Param::PARAM_INPUT, $cdfilial,              Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_SERVICOS',      Param::PARAM_INPUT, $row['LISTASERV'],      Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_DTINICIAL',     Param::PARAM_INPUT, $dateInicial,           Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_DTFINAL',       Param::PARAM_INPUT, $dateFinal,             Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_IDARQUIVO',     Param::PARAM_INPUT, $idArquivo,             Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_IDPRODUTOS',    Param::PARAM_INPUT, $idProdutos,            Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_IDSEPPRATO',    Param::PARAM_INPUT, $idSepPrato,            Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_PEACRESCIMO',   Param::PARAM_INPUT, $peAcrescimo,           Param::PARAM_TYPE_INT));
        $this->procedureConsumoPlanejado->addParam(new Param('P_IDARREDOND',    Param::PARAM_INPUT, $idarredond,            Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_NRCONSPLAN',    Param::PARAM_INPUT, $nrConsPlan,            Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_CDPRODUTOINI',  Param::PARAM_INPUT, $cdProdutoIni,          Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_CDPRODUTOFIN',  Param::PARAM_INPUT, $row['P_CDPRODUTOFIN'], Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_CDOPERADOR',    Param::PARAM_INPUT, $cdoperador,            Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_STDTINICIAL',   Param::PARAM_INPUT, $dateInicialFormatted,  Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_STDTFINAL',     Param::PARAM_INPUT, $dateFinalFormatted,    Param::PARAM_TYPE_STR));

        $resultado         = $this->procedureConsumoPlanejado->execute();
        $novoCodigoLista   = $this->novoCodigo->getCodeByTableName('LISTA', $nrorg);
        $row['NRCONSPLAN'] = $nrConsPlan;
        $row['NRLISTA']    = $novoCodigoLista;
        $this->nrorg       = $nrorg;
        $this->cdfilial    = $cdfilial;
        $this->nrConsPlan  = $nrConsPlan;
    }
    
    public function afterProcessReport(DTO\Request\Row $request, DTO\Response $response, $report) {
        $parametros = 
        array(
            "NRORG"      => $this->nrorg, 
            "CDFILICPLA" => $this->cdfilial, 
            "NRCONSPLAN" => $this->nrConsPlan, "__is_new" => false
        );
        $newRequest  = new DTO\Request\Row($parametros, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        $newResponse = new DTO\Response();
        $this->consplan->delete($newRequest, $newResponse);
    }    
}